==> certi-3.2.1.tar.gz: certi-3.2.1/libCERTI/LBTS.cc <==
// ----------------------------------------------------------------------------
// CERTI - HLA RunTime Infrastructure
// Copyright (C) 2002, 2003, 2004  ONERA
//
// This file is part of CERTI-libCERTI
//
// CERTI-libCERTI is free software ; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License
// as published by the Free Software Foundation ; either version 2 of
// the License, or (at your option) any later version.
//
// CERTI-libCERTI is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY ; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this program ; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA
//
// $Id: LBTS.cc,v 3.5 2004/05/17 22:10:10 breholee Exp $
// ----------------------------------------------------------------------------

#include <config.h>
#include "LBTS.hh"

#include "PrettyDebug.hh"

#include <float.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <stdlib.h>

using std::vector ;  

namespace certi {

static pdCDebug D("LBTS", __FILE__);

// ----------------------------------------------------------------------------
/** Constructor.  LBTS is set to infinite in case of constrained
  federate without any regulating federate.
*/
LBTS::LBTS()
    : _LBTS(DBL_MAX), MyFederateNumber(0)
{
}

// ----------------------------------------------------------------------------
LBTS::~LBTS()
{
}

// ----------------------------------------------------------------------------
//! Compute the LBTS.
void
LBTS::compute()
{
    FederationTime hl ;

    // LBTS = + l'infini
    _LBTS = DBL_MAX ;

    ClockSet::iterator i ;
    for (i = clocks.begin(); i != clocks.end(); ++i) {
        if (i->first != MyFederateNumber) {
            hl = i->second ;
            if (hl < _LBTS)
                _LBTS = hl ;
        }
    }
}

// ----------------------------------------------------------------------------
/** Check if a federate exists
 */
bool
LBTS::exists(FederateHandle federate) const
{
    return clocks.find(federate) != clocks.end();
}

// ----------------------------------------------------------------------------
/** Get all the federate handle and time in a list of pairs
    @param l Output parameter, list receiving values
*/
void
LBTS::get(vector<FederateClock> &v) const
{
    v.reserve(v.size() + clocks.size());
    std::copy(clocks.begin(), clocks.end(), std::back_inserter(v));
}

// ----------------------------------------------------------------------------
/** Insert a new federate and time
 */
void
LBTS::insert(FederateHandle num_fed, FederationTime time)
{
    // TODO: is this check really required ?
    if (exists(num_fed))
        throw RTIinternalError("LBTS: Federate already present.");

    // BUG: We should verify that clock time is correct.
    clocks[num_fed] = time ;
    compute();
}

// ----------------------------------------------------------------------------
//! update a federate
void
LBTS::update(FederateHandle num_fed, FederationTime time)
{
    D.Out(pdDebug, "LBTS.update: Updating federate %d(%f).", num_fed, time);

    ClockSet::iterator it = clocks.find(num_fed);

    if (it == clocks.end())
        throw RTIinternalError("LBTS: Federate not found.");

    // Coherence test.
    if (it->second > time)
        D.Out(pdDebug,
              "LBTS.update: federate %u, new time lower than oldest one.",
              num_fed);
    else {
        D.Out(pdDebug, "before LBTS.update: federate %u, old time %f.",
              it->first, it->second);
        it->second = time ;
        D.Out(pdDebug, "after LBTS.update: federate %u, new time %f.",
              it->first, it->second) ;
        compute();
    }
}

// ----------------------------------------------------------------------------
//! Remove a federate
void
LBTS::remove(FederateHandle num_fed)
{
    ClockSet::iterator it = clocks.find(num_fed);

    if (it == clocks.end())
        throw RTIinternalError("LBTS: Federate not found.");

    clocks.erase(it);
    compute();
}

} // namespace certi

// $Id: LBTS.cc,v 3.5 2004/05/17 22:10:10 breholee Exp $

==> certi-3.2.4.tar.gz: certi-3.2.4/libCERTI/LBTS.cc <==
// ----------------------------------------------------------------------------
// CERTI - HLA RunTime Infrastructure
// Copyright (C) 2002, 2003, 2004  ONERA
//
// This file is part of CERTI-libCERTI
//
// CERTI-libCERTI is free software ; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License
// as published by the Free Software Foundation ; either version 2 of
// the License, or (at your option) any later version.
//
// CERTI-libCERTI is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY ; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this program ; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA
//
// $Id: LBTS.cc,v 3.6 2007/05/03 15:46:31 rousse Exp $
// ----------------------------------------------------------------------------

#include <config.h>
#include "LBTS.hh"

#include "PrettyDebug.hh"

#include <float.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <stdlib.h>

using std::vector ;  

namespace certi {

static pdCDebug D("LBTS", __FILE__);

// ----------------------------------------------------------------------------
/** Constructor.  LBTS is set to infinite in case of constrained
  federate without any regulating federate.
*/
LBTS::LBTS()
    : _LBTS(DBL_MAX), MyFederateNumber(0)
{
}

// ----------------------------------------------------------------------------
LBTS::~LBTS()
{
}

// ----------------------------------------------------------------------------
//! Compute the LBTS.
void
LBTS::compute()
{
    FederationTime hl ;

    // LBTS = + l'infini
    _LBTS = DBL_MAX ;

    ClockSet::iterator i ;
    for (i = clocks.begin(); i != clocks.end(); ++i) {
        if (i->first != MyFederateNumber) {
            hl = i->second ;
            if (hl < _LBTS)
                _LBTS = hl ;
        }
    }
}

// ----------------------------------------------------------------------------
/** Check if a federate exists
 */
bool
LBTS::exists(FederateHandle federate) const
{
    return clocks.find(federate) != clocks.end();
}

// ----------------------------------------------------------------------------
/** Get all the federate handle and time in a list of pairs
    @param v Output parameter, list receiving values
*/
void
LBTS::get(std::vector<FederateClock> &v) const
{
    v.reserve(v.size() + clocks.size());
    std::copy(clocks.begin(), clocks.end(), std::back_inserter(v));
}

// ----------------------------------------------------------------------------
/** Insert a new federate and time
 */
void
LBTS::insert(FederateHandle num_fed, FederationTime time)
{
    // TODO: is this check really required ?
    if (exists(num_fed))
        throw RTIinternalError("LBTS: Federate already present.");

    // BUG: We should verify that clock time is correct.
    clocks[num_fed] = time ;
    compute();
}

// ----------------------------------------------------------------------------
//! update a federate
void
LBTS::update(FederateHandle num_fed, FederationTime time)
{
    D.Out(pdDebug, "LBTS.update: Updating federate %d(%f).", num_fed, time);

    ClockSet::iterator it = clocks.find(num_fed);

    if (it == clocks.end())
        throw RTIinternalError("LBTS: Federate not found.");

    // Coherence test.
    if (it->second > time)
        D.Out(pdDebug,
              "LBTS.update: federate %u, new time lower than oldest one.",
              num_fed);
    else {
        D.Out(pdDebug, "before LBTS.update: federate %u, old time %f.",
              it->first, it->second);
        it->second = time ;
        D.Out(pdDebug, "after LBTS.update: federate %u, new time %f.",
              it->first, it->second) ;
        compute();
    }
}

// ----------------------------------------------------------------------------
//! Remove a federate
void
LBTS::remove(FederateHandle num_fed)
{
    ClockSet::iterator it = clocks.find(num_fed);

    if (it == clocks.end())
        throw RTIinternalError("LBTS: Federate not found.");

    clocks.erase(it);
    compute();
}

} // namespace certi

// $Id: LBTS.cc,v 3.6 2007/05/03 15:46:31 rousse Exp $
